/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2010
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#ifndef YAYI_ARITHMETIC_T_HPP__
#define YAYI_ARITHMETIC_T_HPP__

/*!@file
 * @Definition of arithmetic operators
 */



#include <functionnal>


#include "Yayi/core/yayiImageCore/include/ApplyToImage_T.hpp"

namespace yayi
{
	namespace arithmetic_pixel
	{

    //! Negate a pixel, returns a new value
    template <class U, class V = U>
    struct operator_negate : public std::unary_function<U, V>
    {
      typedef operator_type_unary operator_tag;
      result_type operator()(const U &u) const throw()
      {
        return !u;
      }
      
    };
    
    
    /*! Addition of a constant to the pixel, return to a new value
     *  @addtogroup unary_op
     */
    template <class U, class V = U>
    struct operator_constant_add : public std::unary_function<U, V>
    {
      typedef operator_type_unary operator_tag;
      const U value;
      operator_constant_add(const U& v) : value(v) {}
      result_type operator()(const U &u) const throw()
      {
        return u + value;
      }
      
    };
    
    



    /*! Binary addition of two pixels, return to a new pixel
     *  @addtogroup binary_op
     */
		template <class U, class V, class W = yayi::supertype<U,V>::type>
			struct operator_add : public std::binary_function<U, V, W>
		{
			result_type operator()(const U &u, const V &v) const throw()
			{
				return u + v;
			}

		};






	} // namespace arithmetic


	namespace arithmetic_operators
	{
    using namespace arithmetic_pixel;




	}


  namespace arithmetic_processors
  {



  }




} // namespace yayi

#endif /* YAYI_ARITHMETIC_T_HPP__ */