/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2010
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#ifndef YAYI_COMMON_HISTOGRAM__HPP___
#define YAYI_COMMON_HISTOGRAM__HPP___

#include <map>
#include <functional>
#include <vector>
#include <algorithm>




/*! @file
 *  @brief Histogram generic definition and manipulation
 */


namespace yayi
{

  /*!@namespace
   * @brief Histograms related structures and definitions
   *
   */
  namespace histograms
  {
    typedef yaINT32       discrete_histogram_bin_type;          //!< Bin type for the discrete histogram case
    typedef yaF_double    continuous_histogram_bin_type;        //!< Bin type for the "continuous" histogram case

    template <class bin_type_t = continuous_histogram_bin_type>
      struct s_histogram_representation_strategy
    {
      typedef continuous_histogram_bin_type         bin_type;
      typedef yaF_double                            count_type;
      typedef std::map<bin_type, count_type>        representation_type;
    };


    template <>
      struct s_histogram_representation_strategy<discrete_histogram_bin_type>
    {
      typedef discrete_histogram_bin_type           bin_type;
      typedef yaF_double                            count_type;
      typedef std::map<bin_type, count_type>        representation_type;
    };


    /*! @brief Generic histogram structure
     *
     * @author Raffi Enficiaud
     */
    template <
      typename bin_type_t, 
      typename representation_type_t = typename s_histogram_representation_strategy<bin_type_t>::representation_type>
    struct s_histogram_t : public representation_type_t
    {
    private:
      typedef representation_type_t                       representation_type;


    public:
      typedef typename representation_type::bin_type      bin_type;         //! Representation type of the bins
      typedef typename representation_type::count_type    count_type;       //! Representation type of the counted elements

      s_histogram_t() : representation_type_t() {}
      
      bin_type max_bin() const throw(){return this->min_element();}
      bin_type min_bin() const throw(){return this->min_element();}

      typename s_sum_supertype<count_type>::type sum() const
      {
        typedef std::plus<typename s_sum_supertype<count_type>::type> op_type;
        return std::accumulate(this->begin(), this->end(), 0, op_type);
      }


    };

  } // namespace histograms

} // namespace yayi

#endif /* YAYI_COMMON_HISTOGRAM__HPP___ */
