/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2010
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#ifndef YAYI_APPLY_TO_IMAGE_TERNARY_HPP__
#define YAYI_APPLY_TO_IMAGE_TERNARY_HPP__

/*!@file
 * This file defines the ternary operators on images.
 * @todo: add multithreading
 */

#include <Yayi/core/yayiImageCore/include/ApplyToImage_T.hpp>


namespace yayi
{

  //! Ternary operator helper
  //! The images structure is different, and no factorisation of the iterator can be performed.
  //! We iterate over the two images, until one of them ends. 
  template <>
  struct s_apply_op_range<iterators_independant_tag, operator_type_ternary_no_return>
  {
    template <class op_, class iter1, class iter2, class iter3, class image1, class image2, class image3>
    yaRC operator()(
      op_& op, 
      iter1 it1, const iter1 it1e, 
      iter2 it2, const iter2 it2e, 
      iter3 it3, const iter3 it3e, 
      image1&, image2&, image3&)
    {
      for(; it1 != it1e && it2 != it2e && it3 != it3e; ++it1, ++it2, ++it3){
        op(*it1, *it2, *it3);
      }
      return yaRC_ok;
    }
  };


  //! The images structures are different, and no factorisation of the iterator can be performed.
  //! We iterate over the two images, until one of them ends. 
  template <>
  struct s_apply_op_range<iterators_independant_tag, operator_type_ternary>
  {
  
    //! Template for operators of the type result = op(operand1, operand2, operand3) (not necessarilly symetric nor commutative etc.)
    template <class op_, class iter1, class iter2, class iter3, class iter4, class image1, class image2, class image3, class image4>
    yaRC operator()(
      op_& op, 
      iter1 it1, const iter1 it1e, 
      iter2 it2, const iter2 it2e, 
      iter3 it3, const iter3 it3e, 
      iter4 it4, const iter4 it4e, 
      image1&, image2&, image3&, image4&)
    {
      for(; it1 != it1e && it2 != it2e && it3 != it3e && it4 != it4e; ++it1, ++it2, ++it3, ++it4){
        *it4 = op(*it1, *it2, *it3);
      }
      return yaRC_ok;
    }

    //! Template for operators of the type op(operand1, operand2, operand3) (not necessarilly symetric)
    template <class op_, class iter1, class iter2, class iter3, class image1, class image2, class image3>
    yaRC operator()(
      op_& op, 
      iter1 it1, const iter1 it1e, 
      iter2 it2, const iter2 it2e, 
      iter3 it3, const iter3 it3e, 
      image1&, image2&, image3&)
    {
      for(; it1 != it1e && it2 != it2e && it3 != it3e; ++it1, ++it2, ++it3){
        op(*it1, *it2, *it3);
      }
      return yaRC_ok;
    }

  };

}


#endif /* YAYI_APPLY_TO_IMAGE_TERNARY_HPP__ */