/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2010
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#ifndef YAYI_CHANNELS_PROCESS_HPP__
#define YAYI_CHANNELS_PROCESS_HPP__

/*!@file
 * This file contains the declaration for channel processing on "multispectral" images
 * @author Raffi Enficiaud
 */

#include <Yayi/core/yayiPixelProcessing/yayiPixelProcessing.hpp>
#include <Yayi/core/yayiImageCore/include/yayiImageCore.hpp>



namespace yayi {

  /*!@brief Copy one channel of the input image into another channel of another image (potentially the same image)
   *
   * @author Raffi Enficiaud
   */
  YPix_ yaRC CopyOneChannelIntoAnother(const IImage* imin, const unsigned int channel_input, const unsigned int channel_output, IImage* imout);


  /*!@brief Copy one channel of the input image into another image which can accept a scalar representation of the pixel of the first image.
   *
   * @author Raffi Enficiaud
   */
  YPix_ yaRC CopyOneChannel(const IImage* imin, const unsigned int channel_input, IImage* imout);


  /*!@brief Split the color input image into the first three components (images should be of the same size).
   *
   * @author Raffi Enficiaud
   */
  YPix_ yaRC CopySplit(const IImage* imin, IImage* imo1, IImage* imo2, IImage* imo3);

}


#endif

