/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2010
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#include "main.hpp"
#include "Yayi/core/yayiImageCore/include/yayiImageCore_Impl.hpp"
#include "Yayi/core/yayiImageCore/include/yayiImageUtilities_T.hpp"


class coordinate_test 
{
public:

  void test_construction()
  {
    BOOST_CHECKPOINT("test_construction");	
    typedef yayi::s_coordinate<2> coordinate_type;

    coordinate_type::scalar_coordinate_type s1[] = {3,4}, s2[] = {3,4}, s3[] = {5,6};

    coordinate_type 
      coord1(coordinate_type::from_table(s1)), 
      coord2(coordinate_type::from_table(s2)), 
      coord3(coordinate_type::from_table(s3)), 
      coord4(3);

    BOOST_CHECK(coord1 == coord2);
    BOOST_CHECK(coord2 == coord1);
    BOOST_CHECK(coord3 != coord1);
    BOOST_CHECK(coord1 != coord3);
    BOOST_CHECK(coord2 != coord3);
    BOOST_CHECK(coord3 != coord2);

    coord1[1] = 3;
    BOOST_CHECK(coord1 == coord4);
    BOOST_CHECK(coord2 != coord4);

    BOOST_CHECK(total_number_of_points(coord1) == 9);
    BOOST_CHECK(total_number_of_points(coord3) == 30);

  }

  void test_from_table_multiple()
  {
    BOOST_CHECKPOINT("test_from_table_multiple");	
    typedef yayi::s_coordinate<2> coordinate_type;
    static const coordinate_type::scalar_coordinate_type coords [] = {0,0, 1,0, -1,0, 0,1};

    std::vector<coordinate_type> v(coordinate_type::from_table_multiple(coords, coords + sizeof(coords)/sizeof(coords[0])));
    
    BOOST_REQUIRE_MESSAGE(v.size() == 4, "Vector size = " << v.size() << " != 4");
    
    using yayi::c2D;

    BOOST_CHECK(v[0] == c2D(0,0));
    BOOST_CHECK(v[1] == c2D(1,0));
    BOOST_CHECK(v[2] == c2D(-1,0));
    BOOST_CHECK(v[3] == c2D(0,1));
  }


  

  void test_access_methods() 
  {
    BOOST_CHECKPOINT("test_access_methods");

  }
};

void register_coordinate_lifetime_test(test_suite*& test)
{
  boost::shared_ptr<coordinate_test> instance( new coordinate_test );
  test->add( BOOST_CLASS_TEST_CASE( &coordinate_test::test_construction, instance ) );
  test->add( BOOST_CLASS_TEST_CASE( &coordinate_test::test_from_table_multiple, instance) );
  test->add( BOOST_CLASS_TEST_CASE( &coordinate_test::test_access_methods, instance ) );
}


