/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2010
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#include <Yayi/python/yayiCommonPython/include/common_python.hpp>
#include <Yayi/core/yayiCommon/common_coordinates.hpp>

#include <iostream>

struct custom_coordinate_to_python
{

  static PyObject* from_coordinate_to_python(yayi::s_coordinate<0> const& s) {
    using yayi::type;
    PyObject* out_t = PyTuple_New(s.dimension());
    if(out_t == NULL)
      return 0;

    unsigned int i = 0, j = s.dimension();
    for(; i < j; i++) {

      PyObject *p = 0;
      try {
        p = PyLong_FromLong(s[i]);
          
        if(p == NULL /*|| PyTuple_SET_ITEM(out_t, i, p) != 0*/) {
          Py_DECREF(out_t);
          return 0;
        }
        PyTuple_SET_ITEM(out_t, i, p);
        //Py_DECREF(p);
      } catch(yayi::errors::yaException &e) {
        Py_XDECREF(p);
        Py_DECREF(out_t);
        return 0;          
      }
    }
    return out_t;
  }    

  static PyObject* convert(yayi::s_coordinate<0> const& s)
  {
    return from_coordinate_to_python(s);
  }
};


struct custom_coordinate_from_python
{
  custom_coordinate_from_python()
  {
    boost::python::converter::registry::push_back(
      &convertible,
      &construct,
      boost::python::type_id< yayi::s_coordinate<0> >());
  }
  
  static yayi::s_coordinate<0>* do_convert(PyObject* obj_ptr, void * pos)
  {
    using namespace yayi;
    //std::cout << "FROM PYTHON TO COORDINATE !!! (2)" << std::endl;      
    if(PyTuple_Check(obj_ptr))
    {
      int i = 0, j = PyTuple_Size(obj_ptr);
      yayi::s_coordinate<0>* s = new (pos) yayi::s_coordinate<0>();
      s->set_dimension(j);
      for(; i < j; i++)
      {
        //std::cout << "FROM PYTHON TO COORDINATE !!! (2) - b-1" << std::endl; 
        PyObject *p = PyTuple_GetItem(obj_ptr, i);
        if(!PyLong_Check(p) && !PyInt_Check(p)) {
          s->~s_coordinate<0>();
          return 0;
        }
        yaINT32 l = PyLong_AsLong(p);
        //std::cout << "FROM PYTHON TO COORDINATE !!! (2) - b-2" << l << std::endl; 
        if(l == -1) {
          PyObject * po = PyErr_Occurred();
          if(po != NULL && PyErr_ExceptionMatches(PyExc_OverflowError)) {
            yaINT64 ll = PyLong_AsLongLong(obj_ptr);
            (*s)[i] = ll;
          }
        }
        (*s)[i] = l;
          
      }
      return s;
    }
    else if(PyList_Check(obj_ptr))
    {
      int i = 0, j = PyList_Size(obj_ptr);
      yayi::s_coordinate<0>* s = new (pos) yayi::s_coordinate<0>();
      s->set_dimension(j);
      for(; i < j; i++)
      {
        PyObject *p = PyList_GetItem(obj_ptr, i);
        if(!PyLong_Check(p) && !PyInt_Check(p)) {
          s->~s_coordinate<0>();
          return 0;
        }
        yaINT32 l = PyLong_AsLong(p);
        if(l == -1) {
          PyObject * po = PyErr_Occurred();
          if(po != NULL && PyErr_ExceptionMatches(PyExc_OverflowError)) {
            yaINT64 ll = PyLong_AsLongLong(obj_ptr);
            (*s)[i] = ll;
          }
        }
        (*s)[i] = l;
          
      }
      return s;
    }
    
    return 0;
  }

  
  
  

  static void* convertible(PyObject* obj_ptr)
  {
    //std::cout << "FROM PYTHON TO COORDINATE !!! convertible" << std::endl;  
    if(obj_ptr == 0)
      return 0;
    
    if(PyTuple_Check(obj_ptr))
    {
      //std::cout << "FROM PYTHON TO COORDINATE !!! convertible-tuple" << std::endl; 
      for(int i = 0, j = PyTuple_Size(obj_ptr); i < j; i++)
      {
        PyObject *p = PyTuple_GetItem(obj_ptr, i);
        if(!PyLong_Check(p) && !PyInt_Check(p)) {
         //     std::cout << "convertible : not 1" << std::endl; 
          return 0;
        }
      }
      return obj_ptr;
    }
    else if(PyList_Check(obj_ptr))
    {
      //std::cout << "FROM PYTHON TO COORDINATE !!! convertible-list" << std::endl; 
      for(int i = 0, j = PyList_Size(obj_ptr); i < j; i++)
      {
        PyObject *p = PyList_GetItem(obj_ptr, i);
        if(!PyLong_Check(p) && !PyInt_Check(p))
          return 0;
      }
      return obj_ptr;
    }
    
    return 0;
  }

  static void construct(
    PyObject* obj_ptr,
    boost::python::converter::rvalue_from_python_stage1_data* data)
  {
    //std::cout << "FROM PYTHON TO COORDINATE !!! 0" << std::endl;  
    void* storage = ((boost::python::converter::rvalue_from_python_storage< yayi::s_coordinate<0> >*)data)->storage.bytes;
    yayi::s_coordinate<0> *var = do_convert(obj_ptr, storage);
    if (var == 0) boost::python::throw_error_already_set();
    data->convertible = storage;
  }
};


void declare_coordinate()
{
  boost::python::to_python_converter<yayi::s_coordinate<0>, custom_coordinate_to_python>();
  custom_coordinate_from_python();
}
